//+------------------------------------------------------------------+
//|                                                      ComPort.mqh |
//|                                         Copyright 2013, Decanium |
//|                                          http://www.decanium.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Decanium"
#property link      "http://www.decanium.com"


struct TrComPortParameters{
   uint DesiredParams;
   int  BaudRate;             /* Data rate                                    */
   int  DefaultTimeout;       /* Default timeout (in milliseconds)             */
   uchar  ByteSize;           /* Data size(4-8)                                 */
   uchar  StopBits;           /* Number of stop bits
                                (ONESTOPBIT/ONE5STOPBITS/TWOSTOPBITS)               */
   uchar  CheckParity;        /* Parity check(0-no,1-yes)                      */
   uchar  Parity;             /* Parity type
                            (NOPARITY/ODDPARITY/EVENPARITY/MARKPARITY/SPACEPARITY)  */
   uchar  RtsControl;         /* Initial RTS state(see RTS_CONTROL_ constants) */
   uchar  DtrControl;         /* Initial DTR state(see DTR_CONTROL_ constants) */
};
/* flags of changing/requested parameters, combined by OR */
enum TrComPortModifiedParams {
   tcpmpBaudRate=1,          /* data rate            */
   tcpmpDefaultTimeout=2,    /* default timeout      */
   tcpmpByteSize=4,          /* data size              */
   tcpmpStopBits=8,          /* number of stop bits        */
   tcpmpCheckParity=16,      /* parity check          */
   tcpmpParity=32,           /* parity value          */
   tcpmpEnableRtsControl=64, /* initial RTS state    */
   tcpmpEnableDtrControl=128,/* initial DTR state    */
   tcpmpClearHidden=65536    /* clear hidden parameters  */
};
/*---------------------------------------------------------------------------
  flags of clearing queues/buffers, combined by OR */
enum TrComPortFlushFlags {
   tcpffRxAbort = 1,
   tcpffTxAbort = 2,
   tcpffRxClear = 4,
   tcpffTxClear = 8
};
/*-------------------------------------------------------------------------*/
enum TrComPortSetControlFlags {
   tcpscfChangeRtsState = 1,
   tcpscfChangeRtsFlag  = 2,
   tcpscfClearRts       = 2,
   tcpscfSetRts         = 3,
   tcpscfChangeDtrState = 4,
   tcpscfChangeDtrFlag  = 8,
   tcpscfClearDtr       = 8,
   tcpscfSetDtr         = 12
};
/*-------------------------------------------------------------------------*/
enum ComPortList {
   COM1=0,
   COM2,
   COM3,
   COM4,
   COM5,
   COM6,
   COM7,
   COM8  
};
/*-------------------------------------------------------------------------*/
enum BaudRateList {
   _600bps=600,
   _1200bps=1200,
   _2400bps=2400,
   _4800bps=4800,
   _9600bps=9600,
   _14400bps=14400,
   _19200bps=19200,
   _28800bps=28800,
   _38400bps=38400,
   _57600bps=57600,
   _115200bps=115200,
   _230400bps=230400
};
/*-------------------------------------------------------------------------*/
#import "TrComPort.dll"
   int TrComPortOpen(int portnum);
   int TrComPortClose(int portid);   
   int TrComPortSetConfig(int portid, TrComPortParameters& parameters);
   int TrComPortGetConfig(int portid, TrComPortParameters& parameters);
   int TrComPortWriteArray(int portid, uchar& buffer[], uint length, int timeout);
   int TrComPortReadArray(int portid, uchar& buffer[], uint length, int timeout);   
   int TrComPortGetQueue(int portid, uint& input_queue, uint& output_queue);
#import

#define  COM_RX_BUFSIZE 8192// buffer size of the receiver
#define  COM_TX_BUFSIZE 1024// buffer size of the transmitter

string rx_str;
uchar rx_buf[COM_RX_BUFSIZE];//

int rx_cnt;// number of bytes received
int tx_cnt;// number of bytes transmitted
int tx_err;// number of port errors

TrComPortParameters com_par;//structure of COM port parameters
int PortID;//COM port identifier

bool InitComPort()
{//COM port initialization
   rx_cnt=0;
   tx_cnt=0;
   tx_err=0;
   //attempt to open the port
   PortID = TrComPortOpen(inp_com_port_index);
   if(PortID!=inp_com_port_index)
   {
      Print("Error when opening the COM"+DoubleToString(inp_com_port_index+1,0)+" port");
      return(false);
   }
   else   
   {
      Print("The COM"+DoubleToString(inp_com_port_index+1,0)+" port opened successfully");
      //request all parameters, so set all flags
      com_par.DesiredParams = tcpmpBaudRate|tcpmpDefaultTimeout|tcpmpByteSize|tcpmpStopBits|tcpmpCheckParity|tcpmpParity|tcpmpEnableRtsControl|tcpmpEnableDtrControl;
      //read current parameters 
      if(TrComPortGetConfig(PortID, com_par)==-1)
         return(false);//read error
      //
      com_par.ByteSize=8;  //8 bits
      com_par.Parity=0;    //no parity check
      com_par.StopBits=0;  //1 stop bit
      com_par.DefaultTimeout=100;//100 ms timeout
      com_par.BaudRate=inp_com_baudrate;//rate - from the parameters of the Expert Advisor
      //
      if(TrComPortSetConfig(PortID, com_par)==-1)
         return(false);//write error
   }
   return(true);    
}    

void DeinitComPort()
{//close the port
   TrComPortClose(PortID);
}
